module StaticPost
  class Base
    extend ActiveModel::Naming

    class << self
      FRONTMATTER_CONTENT = /^\s*---\n+(.*?)\n+---/
      FRONTMATTER = /^\s*---\n+.*?\n+---/

      def all
        @posts || load_posts
      end

      def post_path(path = nil)
        if path.nil?
          @post_path.nil? ? default_post_path : @post_path
        else
          @post_path = File.expand_path(path)
        end
      end

      def default_post_path
        if defined?(Rails)
          File.expand_path('app/views/posts', Rails.root)
        else
          File.expand_path('.')
        end
      end

      def compile(content)
        @compiler = Redcarpet::Markdown.new(SETTINGS[:renderer])
        @compiler.render(content)
      end

      def load_posts
        @posts = post_files.map do |file, i|
          create_record(file, i)
        end
      end

      def post_files
        Dir.entries("#{@post_path}").select do |file|
          file unless File.directory?(file)
        end
      end

      def create_record(file, id)
        {
          id: id,
          attributes: extract_attributes(file)
        }
      end

      def extract_attributes(file)
        file_content = File.read(file)
        {
          content: post_content(file_content)
        }.merge(extract_frontmatter(file_content))
      end

      def extract_frontmatter(content)
        frontmatter = content.match(FRONTMATTER_CONTENT).captures
        if frontmatter.any?
          attributes = YAML.load(frontmatter.first)
          Hash[attributes.map { |k, v| [k.to_sym, v] }]
        else
          {}
        end
      end

      def post_content(content)
        compile(content.sub(FRONTMATTER, ''))
      end
    end
  end
end
